/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// SymbolFieldListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "tinycad.h"
#include "SymbolFieldListCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSymbolFieldListCtrl

CSymbolFieldListCtrl::CSymbolFieldListCtrl()
{
}

CSymbolFieldListCtrl::~CSymbolFieldListCtrl()
{
}


BEGIN_MESSAGE_MAP(CSymbolFieldListCtrl, CListCtrl)
	//{{AFX_MSG_MAP(CSymbolFieldListCtrl)
	ON_NOTIFY_REFLECT(NM_CLICK, OnClick)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSymbolFieldListCtrl message handlers


void CSymbolFieldListCtrl::AttachList( std::vector<CSymbolField>& fields )
{
	// Set the headings for this control
  	DeleteAllItems();
	DeleteColumn(0);
	DeleteColumn(0);
	InsertColumn(0,"#", LVCFMT_LEFT, 30);
	InsertColumn(1,"Visible", LVCFMT_LEFT, 80);
	InsertColumn(2,"Name", LVCFMT_LEFT, 80);
	InsertColumn(3,"Default", LVCFMT_LEFT, 80);

/*
		int location = InsertItem(GetItemCount()+1, "");
		SetItemText(location, 0, it->first.Mid(1));
		SetItemText(location, 1, it->second);

		it ++;
	  }
 */
}

void CSymbolFieldListCtrl::OnClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// NMITEMACTIVATE *pData = (NMITEMACTIVATE *)pNMHDR;
	const MSG *pMsg = GetCurrentMessage();

	CPoint clk = CPoint( pMsg->pt.x, pMsg->pt.y );
	ScreenToClient(&clk);

	*pResult = 0;

	int col = 1;
	int start_x = GetColumnWidth(0);
	int end_x = start_x+GetColumnWidth(1);

	// Is it the editable column the user is clicking on?
	if (clk.x < start_x || clk.x > end_x)
		return;

	// Search which item it is....
	int item = 0;
	CPoint p;
	GetItemPosition( 0, &p );
	int height = p.y;
	int size = GetItemCount();
	while (item < size)
	{
		// Get the location of this item
		GetItemPosition( item, &p );

		// Is this the item?
		if (p.y < clk.y && p.y + height > clk.y )
			break;

		item ++;
	}
	
	// Did we find this item?
	if (item == size)
		return;

	// Now we can edit it....
	int width = GetColumnWidth(1);
	int x = GetColumnWidth(0);
	int y = p.y;

	// Scrap the previous editor
	finishEdit();

	// Create the new edit cell...
	nItem = item;
	editor = new CEdit;
	editor->Create( WS_VISIBLE|WS_CHILD|WS_BORDER, CRect(x,y,x+width,y+height), this, 101 );
	editor->SetFont( GetFont() );
	editor->SetFocus();
	editor->SetWindowText( GetItemText( item, col ) );
	
	*pResult = 0;
}


void CSymbolFieldListCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	finishEdit();
	CListCtrl::OnLButtonDown(nFlags, point);
}

void CSymbolFieldListCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
	finishEdit();
	CListCtrl::OnRButtonDown(nFlags, point);
}



void CSymbolFieldListCtrl::finishEdit()
{
	if (!editor)
	{
		return;
	}

	// Write back the data
	/*
	CString data;
	editor->GetWindowText( data );
	parent->SpiceData[ editLabel ] = data;
	SetItemText(nItem,1,data );
	*/

	// Remove the editor window
	editor->DestroyWindow();
	delete editor;
	editor = NULL;
}

