/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if !defined(AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_)
#define AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "context.h"
#include "object.h"
#include "Library.h"

class CLibraryFile;
class CTinyCadView;
class CDlgDetailsBox;


// The update hints
#define		DOC_UPDATE_INVALIDATE		100
#define		DOC_UPDATE_INVALIDRECTERASE	101
#define		DOC_UPDATE_INVALIDRECT		102

struct doc_invalidrect : public CObject
{
	CRect r;
	int grow;
};

class CTinyCadDoc : public CDocument
{
protected: // create from serialization only
	DECLARE_DYNCREATE(CTinyCadDoc)

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTinyCadDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	CTinyCadDoc();
	virtual ~CTinyCadDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

	friend	CDlgDetailsBox;
	friend	CDrawingObject;


	/////////////////////////////////////////////////////////////////////////////
	//

	// The current object being used to draw on this object
	//

	CDrawingObject	*edit;			// The object currently being edited

	CDrawingObject 	*NextRepeat;	// The repeat object (NULL if none)
	CDrawingObject 	*LastRepeat;	// The last repeated object (NULL if none)
	CPoint	RepeatDir;				// The direction to move when repeating

	int 	NameDir;				// +ve/-ve name increments when repeating
	int 	PinDir;					// +ve/-ve pin number increments when repeating

	CDrawingObject	*selectable;	// The object that will be selected when the user clicks


	/////////////////////////////////////////////////////////////////////////////
	// Create the options
	//
	Option theOptions;


	/////////////////////////////////////////////////////////////////////////////
	//
	int		part;				// The currently displayed part

	/////////////////////////////////////////////////////////////////////////////
	//

	CDrawingObject *head;			// The first object in this drawing
	CDrawingObject *SelObj; 		// The last selected object

	BOOL named;						// Has the file been named?

	int MaxXSize;					// The size of the page
	int MaxYSize;

	int Undo_Level;					// The current undo level
	int Top_Level;					// The highest undo level in the list
	int Bottom_Level;				// The lowest possible undo level
	BOOL Undo_Inc;  				// Flag to increment on next add
	CDrawingObject *Redo_List;				// The pointer to objects which can be redone (ie have been deleted)

	CString	name;					// The name of the drawing
	CString DateStr;				// The date when this design was last edited/saved
  	CString	Title;					// The title of this design
	CString Author;					// The author of this design
	CString Revision;				// The revision code of this design
	CString DocNumber;				// The document number of this design
	CString Organisation;			// The organisation which designed this design
	CString Sheets;					// The number of sheets in this design
	BYTE	ShowDetails;			// Do we display the details box?


public:
	CDrawingObject* GetSelectable();
	void SetSelectable( CDrawingObject *obj );

	// The current grid settings
	TransformSnap	m_snap;

	// Is this document editing a library?
	virtual bool IsLibInUse( CLibraryFile *lib );

	void SetPageBoundries(PRINTDLG &);		// Set the page boundries from a print dialog
	void SetPageBoundries(CPoint NewSize) { MaxXSize=NewSize.x; MaxYSize=NewSize.y; }
											// Set the page boundries from a CPoint
	CPoint GetPageBoundries() { return CPoint(MaxXSize,MaxYSize); }
											// Return the page boundries in a CPoint
	BOOL IsPortrait() { return MaxXSize < MaxYSize; }
	void Add(CDrawingObject *);						// Add an object to this drawing
	void Undo();
	void Redo();
	int GetUndoLevel() { return Undo_Level; }
	void IncUndoLevel() { Undo_Inc = TRUE; }
	void SwapLists();
	void FlushRedo();						// Remove the Redo Buffer

	void Initalize(CTinyCadView *window);
	CDrawingObject *GetHead() { return head; }		// Return the first object in this drawing
	CDrawingObject *GetSelect() { return SelObj; }
	void SetSelect(CDrawingObject *NewSelObj);
	void SetHead(CDrawingObject *NewHead);
	void DetachSelect() { if (SelObj!=NULL) { head=SelObj->next; SelObj->next=NULL; SelObj=NULL; } }

	CString GetName() { return name; }
	void SetName(CString NewName) { name=NewName; named=1; }

	BOOL GetNamed() { return named; }
	void SetNamed(int NewNamed) { named=NewNamed; }


	void DisplayDetails(Context &);			// Draw the design details box

	void LoadAll(BOOL);
	BOOL Load(CArchive &ar,BOOL Details = FALSE);
	BOOL Save(BOOL = TRUE, BOOL = FALSE);
	void Save(CArchive &,BOOL Details = FALSE, BOOL SaveSelect = FALSE);

	void ResetSymbols();
	void DeleteErrors();
	void TagAllResources();

	void Select(CPoint,CPoint);
	void Select(CDrawingObject *);
	void UnSelect();
	void SelectDelete();
	void SelectMove(CPoint);
	void SelectUndoMove(CPoint);
	BOOL IsSelected() { return (SelObj!=NULL); };

	BOOL CanUndo() { return (Undo_Level!=Top_Level); }
	BOOL CanRedo() { return (Undo_Level!=Bottom_Level); }

	BOOL Sort();		// Perform a single pass of a bubble sort
	BOOL SortName();	// As above, but with different keys
	BOOL SortRef();		// As above, but sort by current references
	BOOL SortLayer();	// As above, but sort by layer

	// Redraw our window
	void Invalidate();
	void InvalidateRect( CRect r, BOOL erase, int grow );

	// Set the next repeat object for the editing view
	void SetRepeat( CDrawingObject *pObject );
	CPoint GetRepeatDir() { return CPoint(RepeatDir.x * m_snap.GetGrid(), RepeatDir.y * m_snap.GetGrid()); }
	int GetNameDir() { return NameDir; }
	int GetPinDir() { return PinDir; }
	void SetRepeatDir(int);	// Change the repeat direction
	void SetNameDir( int i ) { NameDir = i; }
	void SetPinDir( int i ) { PinDir = i; }
	void Repeat();
	BOOL CanRepeat() { return NextRepeat != NULL; }

	// Snap to grid on/oof
	BOOL GetSnapToGrid() { return m_snap.GetGridSnap(); }
	void SetSnapToGrid( BOOL r ) { m_snap.SetGridSnap(r); }

	// Select the editing object
	void SelectObject(CDrawingObject *NewO);

	// Get the current edit object
	CDrawingObject *GetEdit() { return edit; }

	// Edit a library symbol
	virtual BOOL IsEditLibrary() { return FALSE; }

	// Pass to the owner
	void SetScrollCentre(CPoint q);

	// Get/set the current part number
	void SetPart( int i ) 
	{ 
		part = i; 
	}

	int GetPart() 
	{ 
		return part; 
	}

	// No obvious answer to this one, but it is
	// a function that should go in here...
	CSize GetTextExtent(const char *str, int style);

	// Get the current options for this document
	Option *GetOptions() { return &theOptions; }

	// Create an enhanced metafile from this function
	HENHMETAFILE CreateMetafile( CDC &ref_dc );


// Generated message map functions
protected:
	//{{AFX_MSG(CTinyCadDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

// Some function associated with designs but not actually part of the class
void HeaderStamp(CArchive &);
BOOL CheckHeaderStamp(CArchive &);

// The dialog to set the design details
class CDlgDetailsBox : public CDialog {
	CTinyCadDoc *parent;
public:	
	CDlgDetailsBox(CTinyCadDoc *pDesign,CWnd *pParentWnd) : CDialog( IDD_DESIGN, pParentWnd ) { parent = pDesign; }

	virtual void OnOK();
	virtual BOOL OnInitDialog();
};




//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_)
