/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "option.h"
#include "revision.h"
#include "registry.h"
#include "TinyCadView.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "diag.h"





void CEditDlg::Show( CTinyCadDoc *pDesign, CDrawingObject *pObject )
{
	m_pDesign = pDesign;
	m_pObject = pObject;

  if (++opens==1) 
  {
	  g_EditToolBar.setDlg( this );
  }

}

void CEditDlg::Close()
{
  if (--opens==0)
  {
	  g_EditToolBar.unsetDlg();
	  ShowWindow( SW_HIDE );
  }
}


CDrawingObject *CEditDlg::getObject()
{
	
	return m_pObject;
}


void CEditDlg::changeSelected( CDrawingObject *previous, CDrawingObject *pObject )
{
	if (previous == m_pObject)
	{
		m_pObject = pObject;
	}
}

////// The no tool dialog box //////

void CEditDlgNoTool::Create()
{
	CDialog::Create( IDD_NO_TOOL, &g_EditToolBar );
}


////// The text edit dialog box //////

BEGIN_MESSAGE_MAP( CEditDlgTextEdit, CEditDlg )
	ON_EN_CHANGE(TEXTBOX_TEXT, OnChange)
	ON_BN_CLICKED(TEXTBOX_LEFT, OnChange)
	ON_BN_CLICKED(TEXTBOX_UP,OnChange)
END_MESSAGE_MAP()


// Create this window
void CEditDlgTextEdit::Create()
{
	  CDialog::Create( IDD_TEXT, &g_EditToolBar );
}


// Open the dialog window
void CEditDlgTextEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
  stop=TRUE;

	Show( pDesign, pObject );

  SetDlgItemText(TEXTBOX_TEXT,static_cast<CDrawText*>(getObject())->str);
  if (getObject()->GetType()==xLabelEx)
	SetDlgItemText(TEXTBOX_LABEL,"Label Name");
  else 
	SetDlgItemText(TEXTBOX_LABEL,"Text");

  if (static_cast<CDrawText*>(getObject())->dir==3)
	CheckRadioButton(TEXTBOX_LEFT,TEXTBOX_UP,TEXTBOX_LEFT);
  else
	CheckRadioButton(TEXTBOX_LEFT,TEXTBOX_UP,TEXTBOX_UP);	

  stop=FALSE;  

  
}



void CEditDlgTextEdit::OnChange()
{
  char str[SIZESTRING];

  if (stop)
	return;

  getObject()->Display();

  GetDlgItemText(TEXTBOX_TEXT, str, sizeof(str) );
  static_cast<CDrawText*>(getObject())->str = str;
  if (GetCheckedRadioButton(TEXTBOX_LEFT,TEXTBOX_UP)==TEXTBOX_LEFT)
	static_cast<CDrawText*>(getObject())->dir=3;
  else
	static_cast<CDrawText*>(getObject())->dir=0;

  getObject()->Display();
}

void CEditDlgTextEdit::ReFocus()
{
  SetDlgItemText(TEXTBOX_TEXT,static_cast<CDrawText*>(getObject())->str);

  SetFocus();
}


////// The Power editing Dialog //////

BEGIN_MESSAGE_MAP( CEditDlgPowerEdit, CEditDlg )
	ON_EN_CHANGE(POWERBOX_TEXT, OnChange)
	ON_BN_CLICKED(POWERBOX_TOP, OnChange)
	ON_BN_CLICKED(POWERBOX_BOTTOM, OnChange)
	ON_BN_CLICKED(POWERBOX_LEFT, OnChange)
	ON_BN_CLICKED(POWERBOX_RIGHT, OnChange)
	ON_BN_CLICKED(POWERBOX_WAVE, OnChange)
	ON_BN_CLICKED(POWERBOX_CIRCLE, OnChange)
	ON_BN_CLICKED(POWERBOX_ARROW, OnChange)
	ON_BN_CLICKED(POWERBOX_BAR, OnChange)
	ON_BN_CLICKED(POWERBOX_EARTH, OnChange)
END_MESSAGE_MAP()

// Create this window
void CEditDlgPowerEdit::Create()
{
	  CDialog::Create( IDD_POWER, &g_EditToolBar );
}

void CEditDlgPowerEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
  stop=TRUE;
  
	Show( pDesign, pObject );
  SetDlgItemText(POWERBOX_TEXT,static_cast<CDrawPower*>(getObject())->str);
  CheckRadioButton(POWERBOX_TOP,POWERBOX_RIGHT,(static_cast<CDrawPower*>(getObject())->dir)+POWERBOX_TOP);
  CheckRadioButton(POWERBOX_BAR,POWERBOX_EARTH,(static_cast<CDrawPower*>(getObject())->which)+POWERBOX_BAR);

  stop=FALSE;
}


void CEditDlgPowerEdit::OnChange()
{
  char str[SIZESTRING];
  if (stop)
	return;

  getObject()->Display();

  GetDlgItemText(POWERBOX_TEXT, str, sizeof(str) );
  static_cast<CDrawPower*>(getObject())->str = str;

  static_cast<CDrawPower*>(getObject())->dir=GetCheckedRadioButton(POWERBOX_TOP,POWERBOX_RIGHT)-POWERBOX_TOP;
  static_cast<CDrawPower*>(getObject())->which=GetCheckedRadioButton(POWERBOX_BAR,POWERBOX_EARTH)-POWERBOX_BAR;

  getObject()->Display();
}




// The Pin editing Dialog

BEGIN_MESSAGE_MAP( CEditDlgPinEdit, CEditDlg )
	ON_EN_CHANGE(PINBOX_NAME, OnChange)
	ON_EN_CHANGE(PINBOX_NUMBER, OnChange)
	ON_EN_CHANGE(PINBOX_LENGTH, OnChange)
	ON_BN_CLICKED(PINBOX_NAMEON, OnChange)
	ON_BN_CLICKED(PINBOX_NUMBERON, OnChange)
	ON_BN_CLICKED(PINBOX_UP, OnChange)
	ON_BN_CLICKED(PINBOX_DOWN, OnChange)
	ON_BN_CLICKED(PINBOX_LEFT, OnChange)
	ON_BN_CLICKED(PINBOX_RIGHT, OnChange)
	ON_BN_CLICKED(PINBOX_NORMAL, OnChange)
	ON_BN_CLICKED(PINBOX_DOT, OnChange)
	ON_BN_CLICKED(PINBOX_CLOCK, OnChange)
	ON_BN_CLICKED(PINBOX_DCLOCK, OnChange)
	ON_BN_CLICKED(PINBOX_POWER, OnChange)
	ON_BN_CLICKED(PINBOX_HIDDEN, OnChange)
	ON_BN_CLICKED(PINBOX_INPUT, OnChange)
	ON_BN_CLICKED(PINBOX_OUTPUT, OnChange)
	ON_BN_CLICKED(PINBOX_BIDIR, OnChange)
	ON_BN_CLICKED(PINBOX_PASSIVE, OnChange)
	ON_BN_CLICKED(PINBOX_TRISTATE, OnChange)
END_MESSAGE_MAP()

CEditDlgPinEdit::CEditDlgPinEdit()
{
  opens=0;
  stop=TRUE;
}


// Create this window
void CEditDlgPinEdit::Create()
{
	  CDialog::Create( IDD_PIN, &g_EditToolBar );
}

void CEditDlgPinEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
  stop=TRUE;
  
  Show( pDesign, pObject );

  char Buffer[2];
  Buffer[0] = static_cast<CDrawPin*>(getObject())->part+'A';
  Buffer[1] = 0;

  SetDlgItemText(PINBOX_NAME,static_cast<CDrawPin*>(getObject())->str);
  SetDlgItemText(PINBOX_NUMBER,static_cast<CDrawPin*>(getObject())->number);
  SetDlgItemText(PINBOX_PART,Buffer);
  SetDlgItemInt(PINBOX_LENGTH,static_cast<CDrawPin*>(getObject())->length);
  CheckRadioButton(PINBOX_UP,PINBOX_RIGHT,static_cast<CDrawPin*>(getObject())->dir+PINBOX_UP);
  CheckRadioButton(PINBOX_NORMAL,PINBOX_HIDDEN,static_cast<CDrawPin*>(getObject())->which+PINBOX_NORMAL);
  CheckRadioButton(PINBOX_INPUT,PINBOX_PASSIVE,static_cast<CDrawPin*>(getObject())->elec+PINBOX_INPUT);
  CheckDlgButton(PINBOX_NAMEON,  ((static_cast<CDrawPin*>(getObject())->show)&1)!=0);
  CheckDlgButton(PINBOX_NUMBERON,((static_cast<CDrawPin*>(getObject())->show)&2)!=0);

  stop=FALSE;
}



void CEditDlgPinEdit::OnChange()
{
  char str[SIZESTRING];

  if (stop)
	return;

  getObject()->Display();

  // Read in the name
  GetDlgItemText(PINBOX_NAME, str, sizeof(str) );
  static_cast<CDrawPin*>(getObject())->str=str;

  GetDlgItemText(PINBOX_NUMBER, str, sizeof(str) );
  static_cast<CDrawPin*>(getObject())->number=str;

  static_cast<CDrawPin*>(getObject())->show  = IsDlgButtonChecked(PINBOX_NAMEON)!=0 ? 1 : 0;
  static_cast<CDrawPin*>(getObject())->show |= IsDlgButtonChecked(PINBOX_NUMBERON)!=0 ? 2 : 0;

  static_cast<CDrawPin*>(getObject())->dir   = GetCheckedRadioButton(PINBOX_UP,PINBOX_RIGHT)-PINBOX_UP;
  static_cast<CDrawPin*>(getObject())->which = GetCheckedRadioButton(PINBOX_NORMAL,PINBOX_HIDDEN)-PINBOX_NORMAL;
  static_cast<CDrawPin*>(getObject())->elec 	= GetCheckedRadioButton(PINBOX_INPUT,PINBOX_PASSIVE)-PINBOX_INPUT;

  int length = max(5,GetDlgItemInt(PINBOX_LENGTH));
  static_cast<CDrawPin*>(getObject())->length = length;
  getObject()->m_pDesign->GetOptions()->SetPinLength( length );

  getObject()->Display();
}



void CEditDlgPinEdit::ReFocus()
{
  stop = TRUE;

  SetFocus();

  SetDlgItemText(PINBOX_NAME,static_cast<CDrawPin*>(getObject())->str);
  SetDlgItemText(PINBOX_NUMBER,static_cast<CDrawPin*>(getObject())->number);

  stop = FALSE;
}



// The Rotate Dialog

BEGIN_MESSAGE_MAP( CEditDlgRotateBox, CEditDlg )
	ON_BN_CLICKED(ROTATEBLOCK_LEFT, OnLeft)
	ON_BN_CLICKED(ROTATEBLOCK_RIGHT, OnRight)
	ON_BN_CLICKED(ROTATEBLOCK_MIRROR, OnMirror)	
END_MESSAGE_MAP()


// Create this window
void CEditDlgRotateBox::Create()
{
	  CDialog::Create( IDD_ROTATE, &g_EditToolBar );
}

void CEditDlgRotateBox::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{

	Show( pDesign, pObject );
}



void CEditDlgRotateBox::OnLeft()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(3);
  getObject()->Display();
}

void CEditDlgRotateBox::OnRight()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(2);
  getObject()->Display();
}

void CEditDlgRotateBox::OnMirror()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(4);
  getObject()->Display();
}




////// The Line editing Dialog //////

// Create this window
void CEditDlgLineEdit::Create()
{
	  CDialog::Create( IDD_LINE, &g_EditToolBar );
}


void CEditDlgLineEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
	Show( pDesign, pObject );
  	CheckRadioButton(LINEBOX_FREE,LINEBOX_45,angle);
}






////// The Method Edit dialog //////

BEGIN_MESSAGE_MAP( CEditDlgMethodEdit, CEditDlg )
	ON_BN_CLICKED(METHODBOX_MIRROR, OnChangeRotation )
	ON_BN_CLICKED(METHODBOX_UP, OnChangeRotation )
	ON_BN_CLICKED(METHODBOX_DOWN, OnChangeRotation )
	ON_BN_CLICKED(METHODBOX_LEFT, OnChangeRotation )
	ON_BN_CLICKED(METHODBOX_RIGHT, OnChangeRotation )
	ON_BN_CLICKED(METHODBOX_NAMEON, OnChangeText )
	ON_BN_CLICKED(METHODBOX_OTHERON, OnChangeText )
	ON_BN_CLICKED(METHODBOX_REFON, OnChangeText )
	ON_EN_CHANGE(METHODBOX_NAME, OnChangeText )
	ON_EN_CHANGE(METHODBOX_OTHER, OnChangeText )
	ON_EN_CHANGE(METHODBOX_REF, OnChangeText )
	ON_EN_CHANGE(METHODBOX_PPP, OnChangePart )
END_MESSAGE_MAP()

// Create this window
void CEditDlgMethodEdit::Create()
{
	  CDialog::Create( IDD_METHOD, &g_EditToolBar );
}


void CEditDlgMethodEdit::Open(CTinyCadDoc *pDesign,CDrawingObject *pObject)
{

  stop=TRUE;

  Show( pDesign, pObject );

  CString theString = "Symbol - ";
  SetWindowText(theString + static_cast<CDrawMethod*>(getObject())->SymName);

  // Create the part in package string
  char Buffer[2];
  if ((static_cast<CDrawMethod*>(getObject())->GetSymbolData())->ppp>1) 
  {
	Buffer[0] = static_cast<CDrawMethod*>(getObject())->part+'A';
	Buffer[1] = 0;
  } else
	Buffer[0] = 0;

  CheckRadioButton(METHODBOX_UP,METHODBOX_RIGHT,((static_cast<CDrawMethod*>(getObject())->rotate)&3)+METHODBOX_UP);
  CheckDlgButton(METHODBOX_MIRROR,((static_cast<CDrawMethod*>(getObject())->rotate)&4)!=0 ? 1 : 0);
  CheckDlgButton(METHODBOX_NAMEON,static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Name]);
  CheckDlgButton(METHODBOX_OTHERON,static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Other]);
  CheckDlgButton(METHODBOX_REFON,static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Ref]);

  SetDlgItemText(METHODBOX_NAME,static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Name]);
  SetDlgItemText(METHODBOX_OTHER,static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Other]);
  SetDlgItemText(METHODBOX_REF,static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Ref]);
  SetDlgItemText(METHODBOX_PPP,Buffer);

  stop=FALSE;
}


// Update the method after a change in the dialog
void CEditDlgMethodEdit::OnChangeRotation()
{
  if (stop)
	return;

  getObject()->Display();

  // Set the rotation
  static_cast<CDrawMethod*>(getObject())->rotate=GetDir();

  // Update the method's attributes
  static_cast<CDrawMethod*>(getObject())->NewRotation();

  // re-display it
  getObject()->Display();
}


// Update the method after a change in the dialog
void CEditDlgMethodEdit::OnChangeText()
{
  if (stop)
	return;

  getObject()->Display();

  // Set the text strings
  char str[SIZESTRING];
  GetDlgItemText(METHODBOX_NAME, str, sizeof(str) );
  static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Name]=str;
  GetDlgItemText(METHODBOX_OTHER, str, sizeof(str) );
  static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Other]=str;
  GetDlgItemText(METHODBOX_REF, str, sizeof(str) );
  static_cast<CDrawMethod*>(getObject())->Field[CDrawMethod::Ref]=str;

  // Get the bools
  static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Name]=IsDlgButtonChecked(METHODBOX_NAMEON)!=0;
  static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Other]=IsDlgButtonChecked(METHODBOX_OTHERON)!=0;
  static_cast<CDrawMethod*>(getObject())->Show[CDrawMethod::Ref]=IsDlgButtonChecked(METHODBOX_REFON)!=0;

  // re-display it
  getObject()->Display();
}


// Update the method after a change in the dialog
void CEditDlgMethodEdit::OnChangePart()
{
  if (stop)
	return;

  getObject()->Display();

  char str[SIZESTRING];

  GetDlgItemText(METHODBOX_PPP, str, sizeof(str) );
  str[0]=toupper(str[0]);
  // Is this a correct device letter?
  if (str[0]!=0) {
	if (str[1]!=0 || str[0]<'A' || str[0]>'Z') {
		Message(IDS_BADDEVICE,MB_ICONEXCLAMATION);
		str[0]=static_cast<CDrawMethod*>(getObject())->part+'A';
		str[1]=0;
		SetDlgItemText(METHODBOX_PPP,str);
 	} else {
		int NewPart = str[0]-'A';
		// Is this part letter within the parts defined for this symbol
		if (NewPart < (static_cast<CDrawMethod*>(getObject())->GetSymbolData())->ppp)
			static_cast<CDrawMethod*>(getObject())->part=NewPart;
		else
			Message(IDS_BADPART,MB_ICONEXCLAMATION);
  	}
  }
  
  // re-display it
  getObject()->Display();
}




// Get the rotation and mirroring
int CEditDlgMethodEdit::GetDir()
{
  int dir;

  dir=GetCheckedRadioButton(METHODBOX_UP,METHODBOX_RIGHT);

  return dir-METHODBOX_UP+(IsDlgButtonChecked(METHODBOX_MIRROR)!=0 ? 4 : 0);
}




////// The dialog for the BusName edit //////

BEGIN_MESSAGE_MAP( CEditDlgBusNameEdit, CEditDlg )
	ON_EN_CHANGE(TEXTBOX_TEXT, OnChange)
	ON_BN_CLICKED(TEXTBOX_LEFT, OnChange)
	ON_BN_CLICKED(TEXTBOX_UP,OnChange)
END_MESSAGE_MAP()


// Create this window
void CEditDlgBusNameEdit::Create()
{
	  CDialog::Create( IDD_TEXT, &g_EditToolBar );
}


// Open the dialog window
void CEditDlgBusNameEdit::Open(CTinyCadDoc *pDesign,CDrawingObject *pObject)
{
  stop=TRUE;

  Show( pDesign, pObject );

  SetDlgItemText(TEXTBOX_TEXT,static_cast<CDrawBusName*>(getObject())->str);
  SetDlgItemText(TEXTBOX_LABEL,"Bus Name");

  if (static_cast<CDrawBusName*>(getObject())->dir==3)
	CheckRadioButton(TEXTBOX_LEFT,TEXTBOX_UP,TEXTBOX_LEFT);
  else
	CheckRadioButton(TEXTBOX_LEFT,TEXTBOX_UP,TEXTBOX_UP);	

  stop=FALSE;  
}


void CEditDlgBusNameEdit::OnChange()
{
  char str[SIZESTRING];

  if (stop)
	return;

  getObject()->Display();

  GetDlgItemText(TEXTBOX_TEXT, str, sizeof(str) );
  static_cast<CDrawBusName*>(getObject())->str = str;
  if (GetCheckedRadioButton(TEXTBOX_LEFT,TEXTBOX_UP)==TEXTBOX_LEFT)
	static_cast<CDrawBusName*>(getObject())->dir=3;
  else
	static_cast<CDrawBusName*>(getObject())->dir=0;

  getObject()->Display();
}

void CEditDlgBusNameEdit::ReFocus()
{
  SetDlgItemText(TEXTBOX_TEXT,static_cast<CDrawBusName*>(getObject())->str);

  SetFocus();
}

